package be.iminds.ilabt.jfed.rspec;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.*;

/**
 * config for AdvertisementBasedRspecGenerator
 */
public class AdvertisementBasedRspecGeneratorConfig {
    @Nonnull public final List<NodeInfo> nodes;

    @JsonCreator
    public AdvertisementBasedRspecGeneratorConfig(
            @Nonnull @JsonProperty("nodes") List<NodeInfo> nodes) {
        this.nodes = nodes;
    }

    @Nonnull
    public List<NodeInfo> getNodes() {
        return nodes;
    }

    public static class NodeInfo {
        private final int count;
        @Nonnull private final String clientIdBase;               //setter
        @Nullable private final String ansibleGroup;              //setter
        @Nullable private final String diskImage;                 //setter
        @Nullable private final List<String> componentIdSelector; //setter             (how to set componentID)
        @Nullable private final List<String> componentIdFilter;   //filter             (which componentIds are allowed)
        @Nullable private final Boolean componentNameDenyThenAllow; //filter config      (does allow or deny take priority? default = true = deny takes priority)
        @Nullable private final List<String> componentNameAllowRegex;        //filter             (which componentNames are allowed, default = allow all)
        @Nullable private final List<String> componentNameDenyRegex;         //filter             (which componentNames are not allowed, default = deny all)
        @Nullable private final String sliverType;                //filter AND setter
        @Nullable private final List<String> hardwareTypeFilter;  //filter
        @Nullable private final Boolean availableFilter;          //filter
        @Nullable private final Boolean exclusive;                //setter

        public NodeInfo(int count,
                        @Nullable String clientIdBase,
                        @Nullable String ansibleGroup,
                        @Nullable String diskImage,
                        @Nullable List<String> componentIdSelector,
                        @Nullable List<String> componentIdFilter,
                        @Nullable Boolean componentNameDenyThenAllow,
                        @Nullable List<String> componentNameAllowRegex,
                        @Nullable List<String> componentNameDenyRegex,
                        @Nullable String sliverType,
                        @Nullable List<String> hardwareTypeFilter,
                        @Nullable Boolean availableFilter,
                        @Nullable Boolean exclusive) {
            this.count = count;
            this.clientIdBase = clientIdBase == null ? "node" : clientIdBase;
            this.ansibleGroup = ansibleGroup;
            this.diskImage = diskImage;
            this.componentIdSelector = componentIdSelector;
            this.componentIdFilter = componentIdFilter;
            this.hardwareTypeFilter = hardwareTypeFilter;
            this.availableFilter = availableFilter;
            this.sliverType = sliverType;
            this.exclusive = exclusive;
            this.componentNameDenyThenAllow = componentNameDenyThenAllow;
            this.componentNameAllowRegex = componentNameAllowRegex;
            this.componentNameDenyRegex = componentNameDenyRegex;
        }

        @JsonCreator
        public static NodeInfo createNodeInfoFromJsonObject(@Nonnull Map<String,Object> props) {
            //TODO: Check if types are correct etc
            int count = (int) props.get("count");
            List<String> componentIdSelector = null;
            Object componentIdSelectorObj = props.get("componentIdSelector");
            if (componentIdSelectorObj != null) {
                if (componentIdSelectorObj instanceof Collection) {
                    componentIdSelector = new ArrayList((Collection) componentIdSelectorObj);
                }
                if (componentIdSelectorObj instanceof String) {
                    componentIdSelector = Collections.singletonList((String) componentIdSelectorObj);
                }
                assert componentIdSelector != null;
            }
            List<String> componentIdFilter = null;
            Object componentIdFilterObj = props.get("componentIdFilter");
            if (componentIdFilterObj != null) {
                if (componentIdFilterObj instanceof Collection) {
                    componentIdFilter = new ArrayList((Collection) componentIdFilterObj);
                }
                if (componentIdFilterObj instanceof String) {
                    componentIdFilter = Collections.singletonList((String) componentIdFilterObj);
                }
                assert componentIdFilter != null;
            }
            List<String> componentNameDenyRegex = null;
            Object componentNameDenyRegexObj = props.get("componentNameDenyRegex");
            if (componentNameDenyRegexObj != null) {
                if (componentNameDenyRegexObj instanceof Collection) {
                    componentNameDenyRegex = new ArrayList((Collection) componentNameDenyRegexObj);
                }
                if (componentNameDenyRegexObj instanceof String) {
                    componentNameDenyRegex = Collections.singletonList((String) componentNameDenyRegexObj);
                }
                assert componentNameDenyRegex != null;
            }
            List<String> componentNameAllowRegex = null;
            Object componentNameAllowRegexObj = props.get("componentNameAllowRegex");
            if (componentNameAllowRegexObj != null) {
                if (componentNameAllowRegexObj instanceof Collection) {
                    componentNameAllowRegex = new ArrayList((Collection) componentNameAllowRegexObj);
                }
                if (componentNameAllowRegexObj instanceof String) {
                    componentNameAllowRegex = Collections.singletonList((String) componentNameAllowRegexObj);
                }
                assert componentNameAllowRegex != null;
            }
            List<String> hardwareTypeFilter = null;
            Object hardwareTypeFilterObj = props.get("hardwareTypeFilter");
            if (hardwareTypeFilterObj != null) {
                if (hardwareTypeFilterObj instanceof Collection) {
                    hardwareTypeFilter = new ArrayList((Collection) hardwareTypeFilterObj);
                }
                if (hardwareTypeFilterObj instanceof String) {
                    hardwareTypeFilter = Collections.singletonList((String) hardwareTypeFilterObj);
                }
                assert hardwareTypeFilter != null;
            }
            return new NodeInfo(count,
                    (String) props.get("clientIdBase"),
                    (String) props.get("ansibleGroup"),
                    (String) props.get("diskImage"),
                    componentIdSelector,
                    componentIdFilter,
                    (Boolean) props.get("componentNameDenyThenAllow"),
                    componentNameAllowRegex,
                    componentNameDenyRegex,
                    (String) props.get("sliverType"),
                    hardwareTypeFilter,
                    (Boolean) props.get("availableFilter"),
                    (Boolean) props.get("exclusive")
            );
        }

        public int getCount() {
            return count;
        }

        @Nonnull
        public String getClientIdBase() {
            return clientIdBase;
        }

        @Nullable
        public String getAnsibleGroup() {
            return ansibleGroup;
        }

        @Nullable
        public String getDiskImage() {
            return diskImage;
        }

        @Nullable
        public List<String> getComponentIdSelector() {
            return componentIdSelector;
        }

        @Nullable
        public List<String> getComponentIdFilter() {
            return componentIdFilter;
        }

        @Nullable
        public Boolean getComponentNameDenyThenAllow() {
            return componentNameDenyThenAllow;
        }

        public boolean getComponentNameDenyThenAllowOrDefault() {
            return componentNameDenyThenAllow == null ? true : componentNameDenyThenAllow;
        }

        @Nullable
        public List<String> getComponentNameAllowRegex() {
            return componentNameAllowRegex;
        }

        @Nullable
        public List<String> getComponentNameDenyRegex() {
            return componentNameDenyRegex;
        }

        @Nullable
        public String getSliverType() {
            return sliverType;
        }

        @Nullable
        public List<String> getHardwareTypeFilter() {
            return hardwareTypeFilter;
        }

        @Nullable
        public Boolean getAvailableFilter() {
            return availableFilter;
        }

        @Nullable
        public Boolean getExclusive() {
            return exclusive;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof NodeInfo)) return false;
            NodeInfo nodeInfo = (NodeInfo) o;
            return count == nodeInfo.count &&
                    Objects.equals(clientIdBase, nodeInfo.clientIdBase) &&
                    Objects.equals(ansibleGroup, nodeInfo.ansibleGroup) &&
                    Objects.equals(diskImage, nodeInfo.diskImage) &&
                    Objects.equals(componentIdSelector, nodeInfo.componentIdSelector) &&
                    Objects.equals(componentIdFilter, nodeInfo.componentIdFilter) &&
                    Objects.equals(componentNameDenyThenAllow, nodeInfo.componentNameDenyThenAllow) &&
                    Objects.equals(componentNameAllowRegex, nodeInfo.componentNameAllowRegex) &&
                    Objects.equals(componentNameDenyRegex, nodeInfo.componentNameDenyRegex) &&
                    Objects.equals(sliverType, nodeInfo.sliverType) &&
                    Objects.equals(hardwareTypeFilter, nodeInfo.hardwareTypeFilter) &&
                    Objects.equals(availableFilter, nodeInfo.availableFilter) &&
                    Objects.equals(exclusive, nodeInfo.exclusive);
        }

        @Override
        public int hashCode() {
            return Objects.hash(count, clientIdBase, ansibleGroup, diskImage, componentIdSelector, componentIdFilter, componentNameDenyThenAllow, componentNameAllowRegex, componentNameDenyRegex, sliverType, hardwareTypeFilter, availableFilter, exclusive);
        }

        @Override
        public String toString() {
            return "NodeInfo{" +
                    "count=" + count +
                    ", clientIdBase='" + clientIdBase + '\'' +
                    ", ansibleGroup='" + ansibleGroup + '\'' +
                    ", diskImage='" + diskImage + '\'' +
                    ", componentIdSelector=" + componentIdSelector +
                    ", componentIdFilter=" + componentIdFilter +
                    ", componentNameDenyThenAllow=" + componentNameDenyThenAllow +
                    ", componentNameAllowRegex=" + componentNameAllowRegex +
                    ", componentNameDenyRegex=" + componentNameDenyRegex +
                    ", sliverType='" + sliverType + '\'' +
                    ", hardwareTypeFilter=" + hardwareTypeFilter +
                    ", availableFilter=" + availableFilter +
                    ", exclusive=" + exclusive +
                    '}';
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof AdvertisementBasedRspecGeneratorConfig)) return false;

        AdvertisementBasedRspecGeneratorConfig that = (AdvertisementBasedRspecGeneratorConfig) o;

        return nodes.equals(that.nodes);
    }

    @Override
    public int hashCode() {
        return nodes.hashCode();
    }

    @Override
    public String toString() {
        return "AdvertisementBasedRspecGeneratorConfig{" +
                "nodes=" + nodes +
                '}';
    }
}
